@@ -78,9 +78,6 @@ struct _CsmAutostartAppPrivate {
         int                   launch_type;
         GPid                  pid;
         guint                 child_watch_id;
-
-        DBusGProxy           *proxy;
-        DBusGProxyCall       *proxy_call;
 };
 
 enum {
@@ -97,7 +94,10 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 #define CSM_AUTOSTART_APP_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), CSM_TYPE_AUTOSTART_APP, CsmAutostartAppPrivate))
 
-G_DEFINE_TYPE (CsmAutostartApp, csm_autostart_app, CSM_TYPE_APP)
+static void csm_autostart_app_initable_iface_init (GInitableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (CsmAutostartApp, csm_autostart_app, CSM_TYPE_APP,
+    G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, csm_autostart_app_initable_iface_init))
 
 static void
 csm_autostart_app_init (CsmAutostartApp *app)
@@ -343,42 +343,68 @@ gsettings_condition_cb (GSettings  *settings,
         }
 }
 
+static gboolean
+contained (const gchar * const *items,
+           const gchar         *item)
+{
+        while (*items)
+                if (strcmp (*items++, item) == 0)
+                        return TRUE;
+        return FALSE;
+}
+
+static gboolean
+check_gsettings_schema_has_key (GSettingsSchema   *schema,
+                                const gchar *key)
+{
+        gboolean good;
+        gchar **keys;
+
+        keys = g_settings_schema_list_keys (schema);
+        good = contained ((const gchar **) keys, key);
+        g_strfreev (keys);
+
+        return good;
+}
+
 static gboolean
 setup_gsettings_condition_monitor (CsmAutostartApp *app,
                                    const char      *key)
 {
+        GSettingsSchemaSource *source;
+        GSettingsSchema *schema;
         GSettings *settings;
-        const char * const *schemas;
         char **elems;
-        gboolean schema_exists;
-        guint i;
         gboolean retval;
         char *signal;
 
+        retval = FALSE;
+
         elems = g_strsplit (key, " ", 2);
+
         if (elems == NULL)
-                return FALSE;
-        if (elems[0] == NULL || elems[1] == NULL) {
-                g_strfreev (elems);
-                return FALSE;
+                goto out;
+
+        if (elems[0] == NULL || elems[1] == NULL)
+                goto out;
+
+        source = g_settings_schema_source_get_default ();
+
+        schema = g_settings_schema_source_lookup (source, elems[0], TRUE);
+
+        if (schema == NULL)
+                goto out;
+
+        if (!check_gsettings_schema_has_key(schema, elems[1])) {
+                g_warning ("Gsettings key %s %s could not be found!",
+                           elems[0],
+                           elems[1]);
+                goto out;
         }
 
-        schemas = g_settings_list_schemas ();
-        schema_exists = FALSE;
-        for (i = 0; schemas[i] != NULL; i++) {
-                if (g_str_equal (schemas[i], elems[0])) {
-                        schema_exists = TRUE;
-                        break;
-                }
-        }
-
-        if (schema_exists == FALSE) {
-                g_strfreev (elems);
-                return FALSE;
-        }
-
-        settings = g_settings_new (elems[0]);
+        settings = g_settings_new_full (schema, NULL, NULL);
         retval = g_settings_get_boolean (settings, elems[1]);
+        g_settings_schema_unref (schema);
 
         signal = g_strdup_printf ("changed::%s", elems[1]);
         g_signal_connect (G_OBJECT (settings), signal,
@@ -387,6 +413,7 @@ setup_gsettings_condition_monitor (CsmAutostartApp *app,
 
         app->priv->condition_settings = settings;
 
+out:
         g_strfreev (elems);
 
         return retval;
@@ -597,7 +624,9 @@ setup_condition_monitor (CsmAutostartApp *app)
 
         g_free (key);
 
-        /* FIXME: cache the disabled value? */
+        if (disabled) {
+                /* FIXME: cache the disabled value? */
+        }
 }
 
 static gboolean
@@ -706,16 +735,39 @@ load_desktop_file (CsmAutostartApp *app)
         return TRUE;
 }
 
-static void
-csm_autostart_app_set_desktop_filename (CsmAutostartApp *app,
-                                        const char      *desktop_filename)
+static gboolean
+csm_autostart_app_initable_init (GInitable *initable,
+                                 GCancellable *cancellable,
+                                 GError  **error)
 {
-        GError *error;
+        CsmAutostartApp *app = CSM_AUTOSTART_APP (initable);
+
+        g_return_val_if_fail (app->priv->desktop_filename != NULL, FALSE);
 
         if (app->priv->desktop_file != NULL) {
                 egg_desktop_file_free (app->priv->desktop_file);
                 app->priv->desktop_file = NULL;
-                g_free (app->priv->desktop_id);
+        }
+
+        app->priv->desktop_file = egg_desktop_file_new (app->priv->desktop_filename,
+                                                        error);
+        if (app->priv->desktop_file == NULL) {
+                return FALSE;
+        }
+
+        load_desktop_file (app);
+
+        return TRUE;
+}
+
+static void
+csm_autostart_app_set_desktop_filename (CsmAutostartApp *app,
+                                        const char      *desktop_filename)
+{
+        if (app->priv->desktop_file != NULL) {
+                g_clear_pointer (&app->priv->desktop_file, egg_desktop_file_free);
+                g_clear_pointer (&app->priv->desktop_id, g_free);
+                g_clear_pointer (&app->priv->desktop_filename, g_free);
         }
 
         if (desktop_filename == NULL) {
@@ -723,16 +775,7 @@ csm_autostart_app_set_desktop_filename (CsmAutostartApp *app,
         }
 
         app->priv->desktop_id = g_path_get_basename (desktop_filename);
-
-        error = NULL;
-        app->priv->desktop_file = egg_desktop_file_new (desktop_filename, &error);
-        if (app->priv->desktop_file == NULL) {
-                g_warning ("Could not parse desktop file %s: %s",
-                           desktop_filename,
-                           error->message);
-                g_error_free (error);
-                return;
-        }
+        app->priv->desktop_filename = g_strdup (desktop_filename);
 }
 
 static void
@@ -821,16 +864,6 @@ csm_autostart_app_dispose (GObject *object)
                 priv->child_watch_id = 0;
         }
 
-        if (priv->proxy_call != NULL) {
-                dbus_g_proxy_cancel_call (priv->proxy, priv->proxy_call);
-                priv->proxy_call = NULL;
-        }
-
-        if (priv->proxy != NULL) {
-                g_object_unref (priv->proxy);
-                priv->proxy = NULL;
-        }
-
         if (priv->condition_monitor) {
                 g_file_monitor_cancel (priv->condition_monitor);
         }
@@ -969,7 +1002,7 @@ static int
 _signal_pid (int pid,
              int signal)
 {
-        int status = -1;
+        int status;
 
         /* perhaps block sigchld */
         g_debug ("CsmAutostartApp: sending signal %d to process %d", signal, pid);
@@ -1110,21 +1143,19 @@ autostart_app_start_spawn (CsmAutostartApp *app,
 }
 
 static void
-start_notify (DBusGProxy      *proxy,
-              DBusGProxyCall  *call,
+start_notify (GObject *source,
+              GAsyncResult *result,
               CsmAutostartApp *app)
 {
-        gboolean res;
         GError  *error;
 
         error = NULL;
-        res = dbus_g_proxy_end_call (proxy,
-                                     call,
-                                     &error,
-                                     G_TYPE_INVALID);
-        app->priv->proxy_call = NULL;
 
-        if (! res) {
+        g_dbus_connection_call_finish (G_DBUS_CONNECTION (source),
+                                       result,
+                                       &error);
+
+        if (error != NULL) {
                 g_warning ("CsmAutostartApp: Error starting application: %s", error->message);
                 g_error_free (error);
         } else {
@@ -1139,11 +1170,11 @@ autostart_app_start_activate (CsmAutostartApp  *app,
         const char      *name;
         char            *path;
         char            *arguments;
-        DBusGConnection *bus;
+        GDBusConnection *bus;
         GError          *local_error;
 
         local_error = NULL;
-        bus = dbus_g_bus_get (DBUS_BUS_SESSION, &local_error);
+        bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &local_error);
         if (bus == NULL) {
                 if (local_error != NULL) {
                         g_warning ("error getting session bus: %s", local_error->message);
@@ -1167,34 +1198,18 @@ autostart_app_start_activate (CsmAutostartApp  *app,
                                                  CSM_AUTOSTART_APP_DBUS_ARGS_KEY,
                                                  NULL);
 
-        app->priv->proxy = dbus_g_proxy_new_for_name (bus,
-                                                      name,
-                                                      path,
-                                                      CSM_SESSION_CLIENT_DBUS_INTERFACE);
-        if (app->priv->proxy == NULL) {
-                g_set_error (error,
-                             CSM_APP_ERROR,
-                             CSM_APP_ERROR_START,
-                             "Unable to start application: unable to create proxy for client");
-                return FALSE;
-        }
+        g_dbus_connection_call (bus,
+                                name,
+                                path,
+                                CSM_SESSION_CLIENT_DBUS_INTERFACE,
+                                "Start",
+                                g_variant_new ("(s)", arguments),
+                                NULL,
+                                G_DBUS_CALL_FLAGS_NONE,
+                                -1, NULL,
+                                (GAsyncReadyCallback) start_notify, app);
 
-        app->priv->proxy_call = dbus_g_proxy_begin_call (app->priv->proxy,
-                                                         "Start",
-                                                         (DBusGProxyCallNotify)start_notify,
-                                                         app,
-                                                         NULL,
-                                                         G_TYPE_STRING, arguments,
-                                                         G_TYPE_INVALID);
-        if (app->priv->proxy_call == NULL) {
-                g_object_unref (app->priv->proxy);
-                app->priv->proxy = NULL;
-                g_set_error (error,
-                             CSM_APP_ERROR,
-                             CSM_APP_ERROR_START,
-                             "Unable to start application: unable to call Start on client");
-                return FALSE;
-        }
+        g_object_unref (bus);
 
         return TRUE;
 }
@@ -1418,23 +1433,10 @@ csm_autostart_app_peek_autostart_delay (CsmApp *app)
         return aapp->priv->autostart_delay;
 }
 
-static GObject *
-csm_autostart_app_constructor (GType                  type,
-                               guint                  n_construct_properties,
-                               GObjectConstructParam *construct_properties)
+static void
+csm_autostart_app_initable_iface_init (GInitableIface  *iface)
 {
-        CsmAutostartApp *app;
-
-        app = CSM_AUTOSTART_APP (G_OBJECT_CLASS (csm_autostart_app_parent_class)->constructor (type,
-                                                                                               n_construct_properties,
-                                                                                               construct_properties));
-
-        if (! load_desktop_file (app)) {
-                g_object_unref (app);
-                app = NULL;
-        }
-
-        return G_OBJECT (app);
+        iface->init = csm_autostart_app_initable_init;
 }
 
 static void
@@ -1446,7 +1448,6 @@ csm_autostart_app_class_init (CsmAutostartAppClass *klass)
         object_class->set_property = csm_autostart_app_set_property;
         object_class->get_property = csm_autostart_app_get_property;
         object_class->dispose = csm_autostart_app_dispose;
-        object_class->constructor = csm_autostart_app_constructor;
 
         app_class->impl_is_disabled = is_disabled;
         app_class->impl_is_conditionally_disabled = is_conditionally_disabled;
@@ -1486,10 +1487,19 @@ CsmApp *
 csm_autostart_app_new (const char *desktop_file)
 {
         CsmAutostartApp *app;
+        GError *error;
 
-        app = g_object_new (CSM_TYPE_AUTOSTART_APP,
-                            "desktop-filename", desktop_file,
-                            NULL);
+        error = NULL;
+
+        app = g_initable_new (CSM_TYPE_AUTOSTART_APP,
+                              NULL, &error,
+                              "desktop-filename", desktop_file,
+                              NULL);
+
+        if (error != NULL) {
+                g_warning ("Could not read %s: %s", desktop_file, error->message);
+                g_clear_error (&error);
+        }
 
         return CSM_APP (app);
 }
